import unittest
from structura import Queue


class TestQueue(unittest.TestCase):
    def test_queue_enqueue(self):
        q = Queue(1000)
        for i in range(1000):
            q.enqueue(f'qwerqwer[{i}]]')
        self.assertEqual(q.size, 1000)

    def test_queue_peek(self):
        q = Queue(32)
        for i in range(32):
            q.enqueue(f'qwerqwer[{i}]]')
        self.assertEqual(q.peek(), 'qwerqwer[0]]')

    def test_queue_dequeue(self):
        q = Queue(32)
        for i in range(32):
            q.enqueue(f'qwerqwer[{i}]]')
        self.assertEqual(q.dequeue(), 'qwerqwer[0]]')
        self.assertEqual(q.size, 31)

    def test_queue_is_empty(self):
        q = Queue(32)
        self.assertTrue(q.is_empty())
        q.enqueue('qwerqwer')
        self.assertFalse(q.is_empty())
        q.dequeue()
        self.assertTrue(q.is_empty())

    def test_queue_is_full(self):
        q = Queue(32)
        self.assertFalse(q.is_full())
        for i in range(32):
            q.enqueue(f'qwerqwer[{i}]]')
        self.assertTrue(q.is_full())
        q.dequeue()
        self.assertFalse(q.is_full())

    def test_queue_size(self):
        q = Queue(32)
        self.assertEqual(q.size, 0)
        for i in range(32):
            q.enqueue(f'qwerqwer[{i}]]')
        self.assertEqual(q.size, 32)
        q.dequeue()
        self.assertEqual(q.size, 31)

    def test_queue_dequeue_when_empty(self):
        q = Queue(32)
        self.assertRaises(IndexError, q.dequeue)

    def test_queue_peek_when_empty(self):
        q = Queue(32)
        self.assertRaises(IndexError, q.peek)

    def test_queue_enqueue_when_full(self):
        q = Queue(32)
        for i in range(32):
            q.enqueue(f'qwerqwer[{i}]]')
        self.assertRaises(IndexError, q.enqueue, 'qwerqwer')


if __name__ == '__main__':
    unittest.main()



import unittest
from structura import PriorityQueue


class TestPriorityQueue(unittest.TestCase):
    def test_push(self):
        q = PriorityQueue()
        q.push(1, 1)
        q.push(2, 2)
        q.push(3, 3)
        self.assertEqual(q.size, 3)

    def test_pop(self):
        q = PriorityQueue()
        q.push(1, 12341234)
        q.push(2, 2234234)
        q.push(3, 3434)
        self.assertEqual(q.pop(), (3, 3434))
        self.assertEqual(q.pop(), (2, 2234234))
        self.assertEqual(q.pop(), (1, 12341234))
        self.assertRaises(IndexError, q.pop)

    def test_peek(self):
        q = PriorityQueue()
        q.push(1, 1)
        q.push(2, 2)
        q.push(3, 3)
        self.assertEqual(q.peek(), (1, 1))
        q.pop()
        self.assertEqual(q.peek(), (2, 2))
        q.pop()
        self.assertEqual(q.peek(), (3, 3))
        q.pop()
        self.assertRaises(IndexError, q.peek)

    def test_size(self):
        q = PriorityQueue()
        self.assertEqual(q.size, 0)
        q.push(1, 1)
        self.assertEqual(q.size, 1)
        q.push(2, 2)
        self.assertEqual(q.size, 2)
        q.push(3, 3)
        self.assertEqual(q.size, 3)
        q.pop()
        self.assertEqual(q.size, 2)
        q.pop()
        self.assertEqual(q.size, 1)
        q.pop()
        self.assertEqual(q.size, 0)

    def test_empty(self):
        q = PriorityQueue()
        self.assertTrue(q.is_empty())
        q.push(1, 1)
        self.assertFalse(q.is_empty())
        q.pop()
        self.assertTrue(q.is_empty())

    def test_clear(self):
        q = PriorityQueue()
        q.push(1, 1)
        q.push(2, 2)
        q.push(3, 3)
        q.clear()
        self.assertTrue(q.is_empty())


if __name__ == '__main__':
    unittest.main()



import unittest
from structura import LinkedList


class TestLinkedList(unittest.TestCase):
    def test_append(self):
        ll = LinkedList()
        ll.append(1)
        ll.append(2)
        ll.append(3)
        self.assertEqual(ll.head(), 1)
        self.assertEqual(ll.tail(), 3)
        self.assertEqual(ll.size, 3)

    def test_prepend(self):
        ll = LinkedList()
        ll.prepend(1)
        ll.prepend(2)
        ll.prepend(3)
        self.assertEqual(ll.head(), 3)
        self.assertEqual(ll.tail(), 1)
        self.assertEqual(ll.size, 3)

    def test_pop(self):
        ll = LinkedList()
        ll.append(1)
        ll.append(2)
        ll.append(3)
        self.assertEqual(ll.pop(), 3)
        self.assertEqual(ll.pop(), 2)
        self.assertEqual(ll.pop(), 1)
        self.assertEqual(ll.size, 0)
        self.assertRaises(IndexError, ll.pop)

    def test_popleft(self):
        ll = LinkedList()
        ll.append(1)
        ll.append(2)
        ll.append(3)
        self.assertEqual(ll.popleft(), 1)
        self.assertEqual(ll.popleft(), 2)
        self.assertEqual(ll.popleft(), 3)
        self.assertEqual(ll.size, 0)
        self.assertRaises(IndexError, ll.popleft)

    def test_popat(self):
        ll = LinkedList()
        ll.append(1)
        ll.append(2)
        ll.append(3)
        self.assertEqual(ll.popat(0), 1)
        self.assertEqual(ll.popat(0), 2)
        self.assertEqual(ll.popat(0), 3)
        self.assertEqual(ll.size, 0)
        self.assertRaises(IndexError, ll.popat, 0)

    def test_find(self):
        ll = LinkedList()
        ll.append(1)
        ll.append(2)
        ll.append(3)
        self.assertEqual(ll.find(1), 0)
        self.assertEqual(ll.find(2), 1)
        self.assertEqual(ll.find(3), 2)
        self.assertEqual(ll.find(4), -1)

    def test_size(self):
        ll = LinkedList()
        self.assertEqual(ll.size, 0)
        ll.append(1)
        self.assertEqual(ll.size, 1)
        ll.append(2)
        self.assertEqual(ll.size, 2)
        ll.append(3)
        self.assertEqual(ll.size, 3)
        ll.pop()
        self.assertEqual(ll.size, 2)
        ll.pop()
        self.assertEqual(ll.size, 1)
        ll.pop()
        self.assertEqual(ll.size, 0)

    def test_head(self):
        ll = LinkedList()
        self.assertRaises(IndexError, ll.head)
        ll.append(1)
        self.assertEqual(ll.head(), True)
        ll.append(2)
        self.assertEqual(ll.head(), True)
        ll.append(3)
        self.assertEqual(ll.head(), True)
        ll.pop()
        self.assertEqual(ll.head(), True)
        ll.pop()
        self.assertEqual(ll.head(), True)
        ll.pop()
        # self.assertEqual(ll.head(), None)

    def test_tail(self):
        ll = LinkedList()
        self.assertRaises(IndexError, ll.tail)
        ll.append(1)
        self.assertEqual(ll.tail(), 1)
        ll.append(2)
        self.assertEqual(ll.tail(), 2)
        ll.append(3)
        self.assertEqual(ll.tail(), 3)
        ll.pop()
        self.assertEqual(ll.tail(), 2)
        ll.pop()
        self.assertEqual(ll.tail(), 1)
        ll.pop()
        self.assertRaises(IndexError, ll.tail)

    def test_remove(self):
        ll = LinkedList()
        ll.append(100000)
        ll.append(20001231)
        ll.append(3123123123123)
        self.assertEqual(ll.remove(100000), 100000)
        self.assertEqual(ll.remove(20001231), 20001231)
        self.assertEqual(ll.remove(3123123123123), 3123123123123)
        self.assertEqual(ll.size, 0)
        self.assertRaises(IndexError, ll.remove, 31)


if __name__ == '__main__':
    unittest.main()



import os

# removing this line causes error: invalid command 'bdist_wheel', wtf...
import setuptools
from distutils.core import setup, Extension


module = Extension(
    "structura",
    sources=[
        f"./src/{file}" for file in os.listdir("./src") if file.endswith(".c")],
    include_dirs=['./include'],
    extra_compile_args=["-O3", "-std=c99"],
)


def main():
    setup(name="structura",
          version="0.3.0",
          description="C extension module for common data structures",
          author="alperen serkan aksoz",
          author_email="a.serkanaksoz@gmail.com",
          url="https://github.com/sekomer/structura",
          license="MIT",
          ext_modules=[module],
          platforms=['manylinux1_x86_64'],
          classifiers=[
              "Development Status :: 3 - Alpha",
              "Intended Audience :: Developers",
              "License :: OSI Approved :: MIT License",
              "Operating System :: POSIX :: Linux",
              "Programming Language :: C",
              "Programming Language :: Python :: 3",
              "Topic :: Software Development :: Libraries :: Python Modules",
          ])


if __name__ == "__main__":
    main()



import unittest
from structura import Stack


class TestStack(unittest.TestCase):
    def test_stack_push(self):
        s = Stack(1000)
        for i in range(1000):
            s.push(f'qwerqwer[{i}]]')
        self.assertEqual(s.size, 1000)

    def test_stack_peek(self):
        s = Stack(32)
        for i in range(32):
            s.push(f'qwerqwer[{i}]]')
        self.assertEqual(s.peek(), 'qwerqwer[31]]')

    def test_stack_pop(self):
        s = Stack(32)
        for i in range(32):
            s.push(f'qwerqwer[{i}]]')
        self.assertEqual(s.pop(), 'qwerqwer[31]]')
        self.assertEqual(s.size, 31)

    def test_stack_is_empty(self):
        s = Stack(32)
        self.assertTrue(s.is_empty())
        s.push('qwerqwer')
        self.assertFalse(s.is_empty())
        s.pop()
        self.assertTrue(s.is_empty())

    def test_stack_is_full(self):
        s = Stack(32)
        self.assertFalse(s.is_full())
        for i in range(32):
            s.push(f'qwerqwer[{i}]]')
        self.assertTrue(s.is_full())
        s.pop()
        self.assertFalse(s.is_full())

    def test_stack_size(self):
        s = Stack(32)
        self.assertEqual(s.size, 0)
        for i in range(32):
            s.push(f'qwerqwer[{i}]]')
        self.assertEqual(s.size, 32)
        s.pop()
        self.assertEqual(s.size, 31)

    def test_stack_pop_when_empty(self):
        s = Stack(32)
        self.assertRaises(IndexError, s.pop)

    def test_stack_peek_when_empty(self):
        s = Stack(32)
        self.assertRaises(IndexError, s.peek)

    def test_stack_push_when_full(self):
        s = Stack(32)
        for i in range(32):
            s.push(f'qwerqwer[{i}]]')
        self.assertRaises(IndexError, s.push, 'qwerqwer')


if __name__ == '__main__':
    unittest.main()



import unittest
from structura import RingBuffer


class TestRingBuffer(unittest.TestCase):
    def test_ringbuffer_enqueue(self):
        rb = RingBuffer(100)
        for i in range(1000):
            rb.enqueue(f'qwerqwer[{i}]]')
        self.assertEqual(rb.size, 100)

    def test_ringbuffer_peek(self):
        rb = RingBuffer(32)
        for i in range(32):
            rb.enqueue(f'qwerqwer[{i}]]')
        self.assertEqual(rb.peek(), 'qwerqwer[0]]')

    def test_ringbuffer_dequeue(self):
        rb = RingBuffer(32)
        for i in range(32):
            rb.enqueue(f'qwerqwer[{i}]]')
        self.assertEqual(rb.dequeue(), 'qwerqwer[0]]')
        self.assertEqual(rb.size, 31)

    def test_ringbuffer_is_empty(self):
        rb = RingBuffer(32)
        self.assertTrue(rb.is_empty())
        rb.enqueue('qwerqwer')
        self.assertFalse(rb.is_empty())
        rb.dequeue()
        self.assertTrue(rb.is_empty())

    def test_ringbuffer_is_full(self):
        rb = RingBuffer(32)
        self.assertFalse(rb.is_full())
        for i in range(32):
            rb.enqueue(f'qwerqwer[{i}]]')
        self.assertTrue(rb.is_full())
        rb.dequeue()
        self.assertFalse(rb.is_full())

    def test_ringbuffer_size(self):
        rb = RingBuffer(32)
        self.assertEqual(rb.size, 0)
        for i in range(32):
            rb.enqueue(f'qwerqwer[{i}]]')
        self.assertEqual(rb.size, 32)
        rb.dequeue()
        self.assertEqual(rb.size, 31)

    def test_ringbuffer_dequeue_when_empty(self):
        rb = RingBuffer(32)
        self.assertRaises(IndexError, rb.dequeue)

    def test_ringbuffer_peek_when_empty(self):
        rb = RingBuffer(32)
        self.assertRaises(IndexError, rb.peek)


if __name__ == '__main__':
    unittest.main()



import unittest
from structura import HashMap


class TestHashMap(unittest.TestCase):
    def test_hashmap_put(self):
        hm = HashMap(1000)
        for i in range(1000):
            hm.put(f'qwerqwer[{i}]]', i)
        self.assertEqual(hm.size, 1000)

    def test_load_factor(self):
        hm = HashMap(1000, 0.4)
        for i in range(400):
            hm.put(f'qwerqwer[{i}]]', i)
        self.assertEqual(hm.size, 400)

    def test_hashmap_get(self):
        hm = HashMap()
        for i in range(32):
            hm.put(f'brrrr[{i}]]', i)
        self.assertEqual(hm.get('brrrr[0]]'), 0)

    def test_hashmap_remove(self):
        hm = HashMap()
        for i in range(32):
            hm.put(f'yesyesyes[{i}]]', i)
        self.assertEqual(hm.remove('yesyesyes[0]]'), None)
        self.assertEqual(hm.size, 31)

    def test_hashmap_size(self):
        hm = HashMap()
        self.assertEqual(hm.size, 0)
        for i in range(32):
            hm.put(f'qwerqwer[{i}]]', i)
        self.assertEqual(hm.size, 32)
        hm.remove('qwerqwer[0]]')
        self.assertEqual(hm.size, 31)

    def test_hashmap_remove_when_empty(self):
        hm = HashMap()
        self.assertRaises(KeyError, hm.remove, 'qwerqwer')

    def test_hashmap_get_when_empty(self):
        hm = HashMap()
        self.assertRaises(KeyError, hm.get, 'qwerqwer')

    def test_remove_unhashable(self):
        hm = HashMap()
        self.assertRaises(TypeError, hm.remove, [1, 2, 3])


if __name__ == '__main__':
    unittest.main()
